import asyncio
from .utils import *
from typing import Tuple
from loguru import logger
from nonebot.rule import to_me
from nonebot.matcher import Matcher
from nonebot.permission import SUPERUSER
from nonebot.message import event_preprocessor
from nonebot.exception import IgnoredException
from nonebot.params import CommandArg, RegexGroup, ArgPlainText
from nonebot.plugin.on import on_message, on_notice, on_command, on_regex
from nonebot.adapters.onebot.v11 import (
    Bot,
    Event,
    Message,
    NotifyEvent,
    MessageEvent,
    MessageSegment,
    PokeNotifyEvent,
    GroupMessageEvent,
    PrivateMessageEvent
)


openai_cd_dir = {}  # 用于存放cd时间

# 添加和删除关键词
add_new = on_regex(
    r"^添加关键词\s*(\S+.*?)\s*答\s*(\S+.*?)\s*$",
    flags=re.S,
    block=True,
    priority=11,
    rule=to_me(),
    permission=SUPERUSER,
)
check_new = on_command(
    "查看关键词",
    aliases={"查询关键词"},
    block=True,
    priority=11,
    rule=to_me(),
    permission=SUPERUSER,
)
check_all = on_command(
    "查看所有关键词",
    aliases={"查询所有关键词"},
    block=True,
    priority=11,
    rule=to_me(),
    permission=SUPERUSER,
)
del_new = on_command(
    "删除关键词",
    block=True,
    priority=11,
    rule=to_me(),
    permission=SUPERUSER,
)
del_new_list = on_regex(
    r"^删除关键词\s*(\S+.*?)\s*删\s*(\S+.*?)\s*$",
    flags=re.S,
    block=True,
    priority=10,
    rule=to_me(),
    permission=SUPERUSER,
)
bingchat = on_command("bing", priority=10, block=True)      # bing主要的响应器
reserve = on_command("重置会话", priority=10, block=True)       # 重置会话
su_help = on_command("su_help", priority=10, block=True)        # 消息转发给su
release_count = on_command("释放违规",aliases={"解禁bing"}, permission=SUPERUSER, priority=1, block=True)      # 释放用户违规次数
add_blacklist = on_command("添加黑名单", permission=SUPERUSER, priority=1, block=True)      # 添加黑名单(全局)
del_blacklist = on_command("删除黑名单", permission=SUPERUSER, priority=1, block=True)      # 删除黑名单(全局)
# 优先级99, 条件: 艾特bot就触发
ai = on_message(rule=to_me(), priority=99, block=False)
# 优先级1, 不会向下阻断, 条件: 戳一戳bot触发
poke_ = on_notice(rule=to_me(), block=False)
# 使用openai的接口, 优先级5
openai_text = on_command(
    "求助", aliases={"请问", "帮忙"}, block=True, priority=5, rule=to_me())


@add_new.handle()
async def _(
    matched: Tuple[str, ...] = RegexGroup(),
):
    word1, word2 = matched
    if add_(word1, word2) == "寄":
        await add_new.finish("这个关键词已经记住辣")
    else:
        await add_new.finish("我记住了\n关键词："+word1+"\n回复："+word2)


@check_new.handle()
async def _(
    matcher: Matcher,
    args: Message = CommandArg()
):
    word1 = args.extract_plain_text()
    if word1:
        matcher.set_arg("word", args)


@check_new.got("word", prompt="你要查看哪个关键词（查看所有关键词指令：查看所有关键词）")
async def _(tag: str = ArgPlainText("word")):
    mes = check_(tag)
    if mes == "寄":
        await add_new.finish("抱歉没有记过这个关键词捏，请输入[查询所有关键词]来获取全部关键词")
    else:
        output = text_to_png(mes) # 将文字转换为图片
        await add_new.finish(MessageSegment.image(output))


@check_all.handle()
async def _():
    mes = check_al()
    output = text_to_png(mes)
    await add_new.finish(MessageSegment.image(output))

@del_new.handle()
async def _(args: Message = CommandArg()):
    key = args.extract_plain_text()
    if key=="" or key.isspace():
        await del_new.finish("没有关键词，del失败")
    else:
        try:
            del AnimeThesaurus[key]
            with open(Path(__file__).parent.joinpath('resource/json/data.json'), "w", encoding="utf8") as f_new:
                json.dump(AnimeThesaurus, f_new, ensure_ascii=False, indent=4)
            await del_new.send("已删除该关键词下所有回复~")
        except:
            await del_new.finish("del失败, 貌似没有这个关键词呢")

@del_new_list.handle()
async def _(
    matched: Tuple[str, int] = RegexGroup(),
):
    word1, word2 = matched
    if del_(word1, word2) == "寄":
        await del_new_list.finish("找不到关键词或回复序号，请用查看命令核对")
    else:
        await del_new_list.finish("删除成功~")




@ai.handle()
async def _(event: MessageEvent):
    # 配置私聊不启用后，私聊信息直接结束处理
    if not reply_private and isinstance(event, PrivateMessageEvent):
        return
    # 获取消息文本
    msg = str(event.get_message())
    # 去掉带中括号的内容(去除cq码)
    msg = re.sub(r"\[.*?\]", "", msg)
    # 如果是光艾特bot(没消息返回)或者打招呼的话,就回复以下内容
    if (not msg) or msg.isspace() or msg in [
        "你好啊",
        "你好",
        "在吗",
        "在不在",
        "您好",
        "您好啊",
        "你好",
        "在",
    ]:
        await ai.finish(Message(random.choice(hello__reply)))
    # 获取用户nickname
    if isinstance(event, GroupMessageEvent):
        nickname = event.sender.card or event.sender.nickname
    else:
        nickname = event.sender.nickname
    # 从字典里获取结果
    result = await get_chat_result(msg,  nickname)
    # 如果词库没有结果，则调用api获取智能回复
    if result == None:
        qinyun_url = f"http://api.qingyunke.com/api.php?key=free&appid=0&msg={msg}"
        message = await qinyun_reply(qinyun_url)
        logger.info("来自青云客的智能回复: " + message)
        await ai.finish(message=message)
    await ai.finish(Message(result))


@poke_.handle()
async def _(event: PokeNotifyEvent):
    if event.is_tome:
        probability = random.random() # 生成0-1之间的随机数
        # 33%概率回复莲宝的藏话
        if probability < 0.33:
            # 发送语音需要配置ffmpeg, 这里try一下, 不行就随机回复poke__reply的内容
            try:
                await poke_.send(MessageSegment.record(Path(aac_file_path)/random.choice(aac_file_list)))
            except:
                await poke_.send(message=f"{random.choice(poke__reply)}")
        elif probability > 0.66:
            # 33% 概率戳回去
            await poke_.send(Message(f"[CQ:poke,qq={event.user_id}]"))
        # probability在0.33和0.66之间的概率回复poke__reply的内容
        else:
            await poke_.send(message=f"{random.choice(poke__reply)}")


@openai_text.handle()
async def _(event: MessageEvent, msg: Message = CommandArg()):
    if openai_api_key == "寄":
        # 没有配置openai_api_key
        await openai_text.finish("请先配置openai_api_key")
    prompt = msg.extract_plain_text()                               # 获取文本
    if prompt == "" or prompt == None or prompt.isspace():
        await openai_text.finish("需要提供文本prompt")                  # 没有提供文本

    qid = event.get_user_id()                                       # 获取用户id
    try:
        cd = event.time - openai_cd_dir[qid]
    except KeyError:
        cd = cd_time + 1
    if (
        cd > cd_time
        or event.get_user_id() in nonebot.get_driver().config.superusers
    ):                                                                          # 超过cd时间或者是超级用户
        # 记录cd
        openai_cd_dir.update({qid: event.time})
        await openai_text.send(MessageSegment.text(f"让{Bot_NICKNAME}想想吧..."))        # 发送消息
        loop = asyncio.get_event_loop()                                # 获取事件循环
        try:
            # 开一个不会阻塞asyncio的线程调用get_openai_reply函数
            res = await loop.run_in_executor(None, get_openai_reply, prompt)
        except Exception as e:                                        # 如果出错
            await openai_text.finish("Error: " + str(e))                       # 发送错误信息
        # 发送结果, 长消息根据账号可能会报错风控, 这里try一下, 不行就发送图片
        try:
            await openai_text.send(MessageSegment.text(res), at_sender=True)
        except:
            await openai_text.send("文本消息被风控了, 这里咱尝试把文字写在图片上发送了"+MessageSegment.image(text_to_png(res)))
        # 图片都发不出去那没办法了
    else:
        await openai_text.finish(
            MessageSegment.text(
                f"让{Bot_NICKNAME}的脑子休息一下好不好喵, {cd_time - cd:.0f}秒后才能再次使用"),   # 发送cd时间
            at_sender=True
        )


@bingchat.handle()
async def _(event: MessageEvent, msg: Message = CommandArg()):
    uid = event.get_user_id()     # 获取用户id
    if uid in ban_list:         # 如果用户在黑名单中, 则直接返回
        await bingchat.finish(f"阈值大于{THRESHOLD}, 你已被ban, 请通过bot联系SUPERUSER, su会根据数据库记录的信息自行决定是否清空count, 命令头“su_help”, 后接内容\n别重复发, 要是不断通过bot转发给su, 该用户所有事件将永久禁用, 群聊也会退出", at_sender=True)
    msg: str = msg.extract_plain_text()     # 获取消息
    if cookies == {}:       # 如果cookies为空, 则直接返回
        await bingchat.finish("cookie未设置, 无法访问")
    if (msg.isspace() or msg == ""):        # 如果消息为空, 则直接返回
        await bingchat.finish("请告诉我你要交流什么", at_sender=True)
    if uid not in chat_dict:                # 如果用户不在会话字典中, 则创建会话
        _ = await new_chat_(uid)            # 创建会话
        await bingchat.send("新会话已创建, 请稍等, 当前模式creative, 如要切换对话样式请发送“重置会话”， 并附带参数 balanced 或者 creative 或 precise",at_sender=True)
    bot: Chatbot = chat_dict[uid]["Chatbot"]        # 获取用户的Chatbot
    style: str = chat_dict[uid]["model"]            # 获取用户的对话模式

    try:                                            # 从头try到尾
        data = await bot.ask(prompt=msg, conversation_style=style)      # 获取响应, 这一部分是最耗时的
        if data["item"]["result"]["value"]!="Success":                  # 如果返回的结果不是Success, 则直接返回
            await bingchat.send("返回Error: " + data["item"]["result"]["value"], at_sender=True)
            return
        throttling = data["item"]["throttling"]             # 获取限制信息
        maxConversation = throttling["maxNumUserMessagesInConversation"]            # 获取最大对话数
        currentConversation = throttling["numUserMessagesInConversation"]           # 获取当前对话数
        if len(data["item"]["messages"]) < 2:                                       # 如果返回的消息长度小于2, 那么他没回答你消息, 很可能是被掐断了
            await bingchat.send("Error, 该对话已中断, 可能已被bing切断, 已帮你重置会话", at_sender=True)
            _ = await new_chat_(user_id = uid)             # 重置会话并且return
            return
        if "text" not in data["item"]["messages"][1]:                          # 如果返回的消息中没有text, 那么他没回答你消息, 很可能是你问了敏感问题
            if uid in user_info:                                    # 如果用户在用户信息中, 则将用户的违规次数+1
                user_info[uid]["violation"] += 1
            else:                                        # 如果用户不在用户信息中, 则创建用户信息, 并且将用户的违规次数设置为1
                user_info[uid] = {"violation": 1} 
            save_user_info()                                # 持久存储用户信息
            count = user_info[uid]["violation"]         # 获取用户的违规次数
            if count > THRESHOLD:                       # 如果用户的违规次数大于阈值, 则将用户加入黑名单
                ban_list.append(uid)            
            await bingchat.send(data["item"]["messages"][1]["adaptiveCards"][0]["body"][0]["text"], at_sender=True)     # 发送一条text, 一般是违规内容固定的回答
            await bingchat.send(f"bot好像检测到你问了敏感问题, 已被记录次数, 超过阈值({THRESHOLD})会被禁用该功能请注意当前count = {count}\n如被ban想解封请通过bot转发给SUPERUSER, su会根据数据库记录的信息自行决定是否清空count, 命令头“su_help”, 后接内容\n要是不断通过bot转发给su, 该用户将永久禁用, 群聊也会退出",at_sender=True) # 提示消息
            await push_sql(user_id = uid, user_name = event.sender.nickname, content=msg, isrational=False)   # 将用户的违规信息存入数据库, 并且return
            return
        await push_sql(user_id = uid, user_name = event.sender.nickname, content=msg, isrational=True)      # 将用户的正常信息存入数据库
        if maxConversation > currentConversation:       # 没超过最大上限, 直接发送
            await bingchat.send(data["item"]["messages"][1]["text"] + f"\n\n当前{currentConversation} 共 {maxConversation}", at_sender=True)
        else:                                    # 超过最大上限, 发送并且重置会话
            await bingchat.send(data["item"]["messages"][1]["text"]+ f"\n\n当前{currentConversation} 共 {maxConversation}", at_sender=True) 
            _ = await new_chat_(event.get_user_id())
            await bingchat.send("达到对话上限, 已自动重置会话", at_sender=True)
    except Exception as e:      # 如果出现异常, 则直接发送报错信息, 防止bot无反应
        await bingchat.send("Error: " + str(e), at_sender=True)


@reserve.handle()
async def _(event: MessageEvent,msg: Message = CommandArg()):
    """重置会话"""
    user_id = event.get_user_id()       # 获取用户id
    msg: str = msg.extract_plain_text()     # 获取参数
    if msg not in ["balanced", "creative", "precise"]:      # 如果参数不在这三个中, 则直接重置会话
        _ = await new_chat_(user_id=user_id)    # 重置会话
        await reserve.finish("缺少参数或参数错误, 请使用 balanced 或者 creative 或 precise, 这里只能默认给你重置成creative")
    else:
        await reserve.finish(await new_chat_(user_id, msg))  # 重置会话并且返回结果


@su_help.handle()
async def _(bot:Bot,event: MessageEvent, msg: Message = CommandArg()):
    """转发给su"""
    uid = event.get_user_id()
    msg = msg.extract_plain_text()
    reply = f"来自{uid}的消息: {msg}"
    if isinstance(event,GroupMessageEvent): # 如果是群聊, 则获取群号加到后面
        gid = str(event.group_id)
        reply+=f"\n来自群{gid}"
    su = SU_LIST[0]
    await bot.send_private_msg(user_id=su, message=reply)
    await su_help.finish("已转发给su, 请耐心等待回复, 请勿重复发送, 重复发送直接ban")
    

@release_count.handle()
async def _(msg: Message = CommandArg()):
    """清空用户的违规次数"""
    msg = msg.extract_plain_text()
    if msg not in ban_list:
        await release_count.finish("该用户不存在")
    user_info[msg]["violation"] = 0
    ban_list.remove(msg)
    save_user_info()
    await release_count.finish("已清空该用户的违规次数")


@event_preprocessor
async def event_preblock(event: Event, bot: Bot):
    """事件预处理器，阻断黑名单用户的事件"""
    if isinstance(event, MessageEvent) or isinstance(event, NotifyEvent):
        """阻断黑名单用户的消息事件和通知事件"""
        if not event.get_user_id() in bot.config.superusers:        # 超级用户不受限制
            if event.get_user_id() in blackdata["user"]:            # 如果用户在黑名单中, 则阻断
                blockreason = "user in blacklist"                   # 阻断原因
                logger.info(f'当前事件已阻断，原因：{blockreason}')   # 记录日志
                raise IgnoredException(blockreason)                 # 抛出异常(阻断事件)


@add_blacklist.handle()
async def _(msg: Message = CommandArg()):
    """添加黑名单"""
    msg: str = msg.extract_plain_text()     # 获取参数
    if msg.isspace() or msg == "":      # 如果参数为空, 则返回错误
        await add_blacklist.finish("参数错误")
    if msg.isdigit():               # 如果参数是数字, 则添加到黑名单
        blackdata["user"].append(msg)
    else:                        # 如果参数不是数字, 则返回错误
        await add_blacklist.finish("参数错误")
    await add_blacklist.send(f"id{msg}添加成功")
    with open(Path(__file__).parent.joinpath('blacklist.json'), "w", encoding="utf8") as f_new:
        json.dump(blackdata, f_new, ensure_ascii=False, indent=4)   # 将黑名单数据写入文件




@del_blacklist.handle()
async def _(msg: Message = CommandArg()):
    """删除黑名单"""
    msg: str = msg.extract_plain_text()   # 获取参数
    if msg.isspace() or msg == "":  # 如果参数为空, 则返回错误
        await del_blacklist.finish("参数错误")
    if msg.isdigit():   # 如果参数是数字, 则尝试删除黑名单
        try:    
            blackdata["user"].remove(msg)
        except ValueError:
            value = str(blackdata["user"])
            await del_blacklist.finish("参数错误, 列表中只存在:" + value)
    else:
        await del_blacklist.finish("参数错误")
    await del_blacklist.send("删除成功")
    with open(Path(__file__).parent.joinpath('blacklist.json'), "w", encoding="utf8") as f_new:
        json.dump(blackdata, f_new, ensure_ascii=False, indent=4)

                  


from io import BytesIO
from PIL import Image, ImageDraw, ImageFont


def txt_to_img(text: str, font_size=30, font_path="simsun.ttc") -> bytes:
    text = line_break(text)
    d_font = ImageFont.truetype(font_path, font_size)
    lines = text.count('\n')  # 计算行数
    image = Image.new("L", (LINE_CHAR_COUNT*font_size //
                      2 + 50, font_size*lines+50), "white")
    draw_table = ImageDraw.Draw(im=image)
    draw_table.text(xy=(25, 25), text=text, fill='#000000',
                    font=d_font, spacing=4)  # spacing调节机制不清楚如何计算
    new_img = image.convert("RGB")
    img_byte = BytesIO()
    new_img.save(img_byte, format='PNG')
    binary_content = img_byte.getvalue()
    return binary_content


LINE_CHAR_COUNT = 30*2  # 每行字符数：30个中文字符(=60英文字符)
CHAR_SIZE = 30
TABLE_WIDTH = 4


def line_break(line: str) -> str:
    ret = ''
    width = 0
    for c in line:
        if len(c.encode('utf8')) == 3:  # 中文
            if LINE_CHAR_COUNT == width + 1:  # 剩余位置不够一个汉字
                width = 2
                ret += '\n' + c
            else:  # 中文宽度加2，注意换行边界
                width += 2
                ret += c
        else:
            if c == '\t':
                space_c = TABLE_WIDTH - width % TABLE_WIDTH  # 已有长度对TABLE_WIDTH取余
                ret += ' ' * space_c
                width += space_c
            elif c == '\n':
                width = 0
                ret += c
            else:
                width += 1
                ret += c
        if width >= LINE_CHAR_COUNT:
            ret += '\n'
            width = 0
    if ret.endswith('\n'):
        return ret
    return ret + '\n'





from setuptools import find_packages, setup
name = 'nonebot_plugin_smart_reply'

setup(
    name=name,  
    version='0.0.19',
    author="Special-Week",
    author_email='2385612749@qq.com',
    description="encapsulate logger",
    python_requires=">=3.8.*",
    packages=find_packages(),
    long_description="reply插件",
    url="https://github.com/Special-Week/nonebot_plugin_smart_reply",

    package_data={name: ['resource/json/*', 'resource/audio/*']},

    # 设置依赖包
    install_requires=["openai", "ujson","httpx","loguru","pillow","nonebot2","nonebot-adapter-onebot"],
)


import os
import re
import time
import random
import openai
import nonebot
import sqlite3
from pathlib import Path
from loguru import logger
from EdgeGPT import Chatbot
from httpx import AsyncClient
from .txtToImg import txt_to_img

try:
    import ujson as json
except ModuleNotFoundError:
    import json

config = nonebot.get_driver().config
xiaoai_api_key: str = getattr(config, "xiaoai_apikey", "寄")
Bot_NICKNAME: str = getattr(config, "bot_nickname", "脑积水")
Bot_MASTER: str = getattr(config, "Bot_MASTER", "脑积水")
reply_private: bool = getattr(config, "ai_reply_private", False)
openai_api_key: str = getattr(config, "openai_api_key", "寄")
max_tokens: int = getattr(config, "openai_max_tokens", 1000)
cd_time: int = getattr(config, "openai_cd_time", 60)

# 数据库路径
SQLITT_PATH: str = 'data/smart_reply/req_data.db'
# 会话字典，用于存储会话   {"user_id": {"Chatbot": bot,"model":  balanced or creative or precise}}
chat_dict: dict = {}
# 初始化cookies
try:
    cookies: dict = json.load(open("data/smart_reply/cookie.json", "r", encoding="utf8"))
except:
    logger.info("cookies.json不存在, 初始化失败")
    cookies: dict = {}
# 获取超级用户
SU_LIST: list = list(nonebot.get_driver().config.superusers)



def initSomething() -> None:
    """初始化一些东西"""
    if not os.path.exists("data/smart_reply"):
        os.makedirs("data/smart_reply")            # 创建文件夹
    conn = sqlite3.connect(SQLITT_PATH)         # 数据库初始化
    c = conn.cursor()
    try:
        c.execute(
            "CREATE TABLE main (user_id text, user_name text, content text, time text, isrational bool)")
        conn.commit()
    except:
        logger.info("数据库已存在")
    conn.close()
initSomething()


# 获取挑战敏感问题的次数, 超过阈值会被禁用该功能
THRESHOLD = 5       # 阈值
if os.path.exists("./data/smart_reply/user_info.json"):  # 读取用户数据
    with open("data/smart_reply/user_info.json", "r", encoding="utf-8") as f:
        user_info: dict = json.load(f)
else: 
    user_info: dict = {}
    with open("data/smart_reply/user_info.json", "w", encoding="utf-8") as f:
        json.dump(user_info, f, indent=4)
# json结构  {"user_id": {"violation": 0}}


# 初始化获取violation大于阈值的用户
ban_list: list = []
for user_id, user_data in user_info.items():
    if user_data["violation"] > THRESHOLD:
        ban_list.append(user_id)

# 获取黑名单
blackdata: dict = json.load(open(Path(__file__).parent.joinpath(
    'resource/json/blacklist.json'), "r", encoding="utf8"))


def save_user_info() -> None:
    """保存用户数据"""
    with open("data/smart_reply/user_info.json", "w", encoding="utf-8") as f:
        json.dump(user_info, f, indent=4)

async def new_chat_(user_id: str, style: str = "creative") -> str:
    """重置会话"""
    bot = Chatbot(cookies=cookies)
    chat_dict.update({user_id: {"Chatbot": bot, "model": style}})
    return f"重置会话成功, bot: {str(bot)}, model: {style}"

async def push_sql(user_id: str, user_name, content: str, isrational: bool) -> None:
    """sql插入, 记录用户请求"""
    user_name = user_name.replace("'", "")
    content = content.replace("'", "")
    conn = sqlite3.connect(SQLITT_PATH)
    c = conn.cursor()
    c.execute("INSERT INTO main VALUES (?,?,?,?,?)", (user_id, user_name, content,
              time.strftime("%Y-%m-%d %H:%M:%S", time.localtime()), isrational))
    conn.commit()
    conn.close()


# 载入词库(这个词库有点涩)
AnimeThesaurus = json.load(open(Path(__file__).parent.joinpath(
    'resource/json/data.json'), "r", encoding="utf8"))

# 获取resource/audio下面的全部文件
aac_file_path = os.path.join(os.path.dirname(__file__), "resource/audio")
aac_file_list = os.listdir(aac_file_path)

# hello之类的回复
hello__reply = [
    "你好！",
    "哦豁？！",
    "你好！Ov<",
    f"库库库，呼唤{Bot_NICKNAME}做什么呢",
    "我在呢！",
    "呼呼，叫俺干嘛",
]

# 戳一戳消息
poke__reply = [
    "lsp你再戳？",
    "连个可爱美少女都要戳的肥宅真恶心啊。",
    "你再戳！",
    "？再戳试试？",
    "别戳了别戳了再戳就坏了555",
    "我爪巴爪巴，球球别再戳了",
    "你戳你🐎呢？！",
    f"请不要戳{Bot_NICKNAME} >_<",
    "放手啦，不给戳QAQ",
    f"喂(#`O′) 戳{Bot_NICKNAME}干嘛！",
    "戳坏了，赔钱！",
    "戳坏了",
    "嗯……不可以……啦……不要乱戳",
    "那...那里...那里不能戳...绝对...",
    "(。´・ω・)ん?",
    "有事恁叫我，别天天一个劲戳戳戳！",
    "欸很烦欸！你戳🔨呢",
    "再戳一下试试？",
    "正在关闭对您的所有服务...关闭成功",
    "啊呜，太舒服刚刚竟然睡着了。什么事？",
    "正在定位您的真实地址...定位成功。轰炸机已起飞",
]


async def get_chat_result(text: str, nickname: str) -> str:
    """从字典里返还消息, 抄(借鉴)的zhenxun-bot"""
    if len(text) < 7:
        keys = AnimeThesaurus.keys()
        for key in keys:
            if text.find(key) != -1:
                return random.choice(AnimeThesaurus[key]).replace("你", nickname)


async def qinyun_reply(url: str) -> str:
    """从qinyunke_api拿到消息"""
    async with AsyncClient() as client:
        response = await client.get(url)
        # 这个api好像问道主人或者他叫什么名字会返回私活,这里replace掉部分(这里好丑，不想改了)
        res = response.json()["content"].replace("林欣", Bot_MASTER).replace("{br}", "\n").replace("贾彦娟", Bot_MASTER).replace("周超辉", Bot_MASTER).replace(
            "鑫总", Bot_MASTER).replace("张鑫", Bot_MASTER).replace("菲菲", Bot_NICKNAME).replace("dn", Bot_MASTER).replace("1938877131", "2749903559").replace("小燕", Bot_NICKNAME)
        res = re.sub(u"\\{.*?\\}", "", res)
        # 检查广告, 这个api广告太多了
        if have_url(res):
            res = Bot_NICKNAME + "暂时听不懂主人说的话呢"
        return res


def have_url(s: str) -> bool:
    """判断传入的字符串中是否有url存在(我他娘的就不信这样还能输出广告?)"""
    index = s.find('.')     # 找到.的下标
    if index == -1:         # 如果没有.则返回False
        return False

    flag1 = (u'\u0041' <= s[index-1] <= u'\u005a') or (u'\u0061' <=
                                                       s[index-1] <= u'\u007a')        # 判断.前面的字符是否为字母
    flag2 = (u'\u0041' <= s[index+1] <= u'\u005a') or (u'\u0061' <=
                                                       s[index+1] <= u'\u007a')        # 判断.后面的字符是否为字母
    if flag1 and flag2:     # 如果.前后都是字母则返回True
        return True
    else:               # 如果.前后不是字母则返回False
        return False


def text_to_png(msg: str) -> bytes:
    """文字转png"""
    return txt_to_img(msg)


def get_openai_reply(prompt: str) -> str:
    """从openai api拿到消息"""
    openai.api_key = openai_api_key
    response = openai.Completion.create(
        model="text-davinci-003",
        prompt=prompt,
        temperature=0.5,
        max_tokens=max_tokens,
        top_p=1.0,
        frequency_penalty=0.0,
        presence_penalty=0.0
    )
    res = response.choices[0].text
    # 去除开头的换行符
    while res.startswith("\n"):
        res = res[1:]
    return "\n" + res



def add_(word1: str, word2: str):
    """添加词条"""
    lis = []
    for key in AnimeThesaurus:
        if key == word1:
            # 获取字典内容
            lis = AnimeThesaurus[key]
            # 判断是否已存在问答
            for word in lis:
                if word == word2:
                    return "寄"
    # 判断是否存在关键词
    if lis == []:
        axis = {word1: [word2]}
    else:
        lis.append(word2)
        axis = {word1: lis}
    AnimeThesaurus.update(axis)
    with open(Path(__file__).parent.joinpath('resource/json/data.json'), "w", encoding="utf8") as f_new:
        json.dump(AnimeThesaurus, f_new, ensure_ascii=False, indent=4)


def check_(target: str) -> str:
    """查询关键词下词条"""
    for item in AnimeThesaurus:
        if target == item:
            mes = "下面是关键词" + target + "的全部响应\n\n"
            # 获取关键词
            lis = AnimeThesaurus[item]
            n = 0
            for word in lis:
                n = n + 1
                mes = mes + str(n) + '、'+word + '\n'
            return mes
    return "寄"


def check_al() -> str:
    """查询全部关键词"""
    mes = "下面是全部关键词\n\n"
    for c in AnimeThesaurus:
        mes = mes + c + '\n'
    return mes

def del_(word1: str, word2: int):
    """删除关键词下具体回答"""
    axis = {}
    for key in AnimeThesaurus:
        if key == word1:
            lis = AnimeThesaurus[key]
            word2 = int(word2) - 1
            try:
                lis.pop(word2)
                axis = {word1: lis}
            except:
                return "寄"
    if axis == {}:
        return "寄"
    AnimeThesaurus.update(axis)
    with open(Path(__file__).parent.joinpath('resource/json/data.json'), "w", encoding="utf8") as f_new:
        json.dump(AnimeThesaurus, f_new, ensure_ascii=False, indent=4)